"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const qs = require("qs");
const React = require("react");
const ReactDOM = require("react-dom");
const Rx = require("rxjs");
const connectionParams_1 = require("./connectionParams");
const octoPaging_1 = require("./octoPaging");
const newReviewBox_1 = require("./newReviewBox");
const reviewBox_1 = require("./reviewBox");
/**
 * Client javascript holding events
 */
class OctoClient {
    constructor(options, token, username) {
        /**
         * Section for this bunch of reviews
         */
        this.section = '';
        /**
         * User key identifying the user that has made the review.
         * If empty annonymous is used
         */
        this.userKey = OctoClient.USER_ANNONYMOUS;
        /**
         * Tells if you want to show textarea only when userkey is set.
         */
        this.hideWhenNotLogged = true;
        /**
         * Tells if you want to disable textarea only when userkey is set.
         */
        this.disableWhenNotLogged = true;
        /**
         * Show more approach will render just a button to load more reviews.
         */
        this.showMoreApproach = true;
        /**
         * Date format for reviews
         */
        this.dateFormat = 'DD MMM YYYY';
        this.setText = (value) => {
            this.txt = value;
        };
        this.getText = () => {
            return this.txt || '';
        };
        this.setUserLoggedKey = (userKey) => {
            this.userKey = userKey;
        };
        this.setPageSize = (pageSize) => {
            this.octoPaging.pageSize = pageSize;
        };
        this.setInsertLastReviews = (insertLastReviews) => {
            this.insertLastReviews = insertLastReviews;
        };
        this.setHideWhenNotLogged = (hideWhenNotLogged) => {
            this.hideWhenNotLogged = hideWhenNotLogged;
        };
        this.setDisableWhenNotLogged = (disableWhenNotLogged) => {
            this.disableWhenNotLogged = disableWhenNotLogged;
        };
        /**
         * Generate a html and inserts in the element
         *
         * @param element Html element where the reviews box is placed
         * @param insertLastReviews
         * @param section section where the review belong
         */
        this.insertInto = (element, section) => {
            this._setUpSocketRealTimeRefresh();
            this.section = section;
            this.element = element;
            this._buildComponent(0);
        };
        this._buildComponent = (page) => {
            if (this.insertLastReviews) {
                this.getAnalyzeds(page).subscribe((data) => {
                    this.octoPaging.currentPage = page;
                    this.octoPaging.lastPage = data.totalPages - 1;
                    if (!data.content) {
                        data.content = [];
                    }
                    this.reviews = data.content;
                    ReactDOM.render(React.createElement(reviewBox_1.default, { addMore: this.getMoreReviews, getReviewsForPage: this.getReviewsForPage, paging: this.octoPaging, setText: this.setText, analize: this.analize, data: data.content, showMoreApproach: this.showMoreApproach, dateFormat: this.dateFormat, hideTextArea: this._getHideTextarea(), disableTextArea: this._getDisableTextarea() }), this.element);
                }, error => console.error('Something went wrong', error));
            }
            else {
                ReactDOM.render(React.createElement(newReviewBox_1.default, { parent: undefined, getReviewsForPage: this.getReviewsForPage, disable: this._getDisableTextarea(), hide: this._getHideTextarea(), analize: this.analize, setText: this.setText }), this.element);
            }
        };
        this.getReviewsForPage = (page) => {
            return this.getAnalyzeds(page)
                .retry(1)
                .map((data) => {
                this.octoPaging.currentPage = page;
                this.octoPaging.lastPage = data.totalPages - 1;
                this.reviews = data.content;
                return data;
            });
        };
        this.getMoreReviews = () => {
            if (this.insertLastReviews) {
                if (this.octoPaging.currentPage >= this.octoPaging.lastPage) {
                    throw new Error('No more pages to render');
                }
                return this.getAnalyzeds(this.octoPaging.currentPage + 1)
                    .retry(1)
                    .map((data) => {
                    this.octoPaging.currentPage++;
                    this.octoPaging.lastPage = data.totalPages - 1;
                    this.reviews = this.reviews.concat(data.content);
                    return data;
                });
            }
            else {
                console.error('Trying to add more reviews but insertLastReviews options is disabled');
            }
        };
        this._setUpSocketRealTimeRefresh = () => {
            this.ws = new WebSocket(OctoClient.SOCKET_URL + '/refresh');
            this.ws.onmessage = this._buildComponent.bind(this, 0);
            setTimeout(() => { this._subscriveToServer(OctoClient.USERNAME + '_npm_', this.identifier); }, 1000);
        };
        this._subscriveToServer = (user, identifier) => {
            if (!this.ws) {
                console.error('Web socket not established');
            }
            else {
                this.ws.send((new connectionParams_1.default(user, identifier)).toJson());
            }
        };
        this._getHideTextarea = () => {
            let hide;
            if (this.hideWhenNotLogged) {
                hide = this.userKey === OctoClient.USER_ANNONYMOUS;
            }
            else {
                hide = false;
            }
            return hide;
        };
        this._getDisableTextarea = () => {
            let disable;
            if (this.disableWhenNotLogged) {
                disable = this.userKey === OctoClient.USER_ANNONYMOUS;
            }
            else {
                disable = false;
            }
            return disable;
        };
        this.getAnalyzeds = (page) => {
            return Rx.Observable.ajax.getJSON(`${OctoClient.URL}/reviews/${page}/${this.octoPaging.pageSize}/${this.section}/${OctoClient.USERNAME}/?`
                + qs.stringify({ token: OctoClient.TOKEN }));
        };
        /**
         * Analize the review  in the #this.txt and rebuilds compoenent in the first page.
         */
        this.analize = () => {
            if (this._getDisableTextarea() || this._getHideTextarea()) {
                throw new Error('You can\'t send reviews. Please log in!');
            }
            return Rx.Observable.ajax.getJSON(`${OctoClient.URL}/evaluate/${this.section}/${OctoClient.USERNAME}/?` +
                qs.stringify({ text: this.txt, token: OctoClient.TOKEN, userkey: this.userKey, identifier: this.identifier }))
                .retry(1)
                .map(data => {
                if (data.status === 'OK' && this.insertLastReviews)
                    this._buildComponent(0);
                if (data.status === 'OK' && this.callback) {
                    this.callback.call(this, data);
                }
                return data;
            });
        };
        this.insertLastReviews = options.insertLastReviews;
        this.octoPaging = new octoPaging_1.default(options.pageSize);
        this.identifier = parseInt((Math.random() * 100000).toString());
    }
    /**
     * Add a callback function to execute when text has been analized.
     * The function will receive a json data with the information about the analized.
     *
     * interface AnalizedResponse {
     *  status: string;
     *  message: string;
     *  positiveFound: Founds[];
     *  negativeFound: Founds[];
     *  processedText: string;
     * }
     *
     * @param callbackSuccess
     */
    setCallback(callbackSuccess) {
        this.callback = callbackSuccess;
    }
    setDateFormat(format) {
        this.dateFormat = format;
    }
    setShowMoreApproach(showMoreApproach) {
        this.showMoreApproach = showMoreApproach;
    }
    setOptions(options) {
        if (options.pageSize)
            this.octoPaging.pageSize = options.pageSize;
        if (options.insertLastReviews !== undefined)
            this.insertLastReviews = options.insertLastReviews;
        if (options.userKey)
            this.userKey = options.userKey;
        if (options.hideWhenNotLogged !== undefined)
            this.hideWhenNotLogged = options.hideWhenNotLogged;
        if (options.disableWhenNotLogged !== undefined)
            this.disableWhenNotLogged = options.disableWhenNotLogged;
        if (options.dateFormat !== undefined)
            this.dateFormat = options.dateFormat;
        if (options.showMoreApproach !== undefined)
            this.showMoreApproach = options.showMoreApproach;
    }
}
OctoClient.USER_ANNONYMOUS = 'annonymous';
/**
 * Server Domain
 */
OctoClient.BASE_DOMAIN = 'www.octopinion.co.uk';
/**
* Url for the server
*/
OctoClient.URL = 'https://' + OctoClient.BASE_DOMAIN;
/**
 * Url for socket
 */
OctoClient.SOCKET_URL = 'wss://' + OctoClient.BASE_DOMAIN;
/**
 * Token provided by Octopinion
 */
OctoClient.TOKEN = '<%= token %>';
/**
 * Username in Octopinion
 */
OctoClient.USERNAME = '<%= user %>';
exports.OctoClient = OctoClient;
var octoClient = new OctoClient({ insertLastReviews: true, pageSize: 20, hideWhenNotLogged: true, disableWhenNotLogged: true });
/**
 * Make it available in window
 */
try {
    window.octoClient = octoClient;
}
catch (e) {
    console.warn('Window is not defined');
}
exports.default = OctoClient;
//# sourceMappingURL=octoClient.js.map