import * as qs from 'qs';
import * as React from 'react';
import * as ReactDOM from 'react-dom';
import * as moment from 'moment';
import * as Rx from 'rxjs';

import ConnectionParams from './connectionParams';
import OctoPaging from './octoPaging';
import NewReviewBox from './newReviewBox';
import ReviewBox from './reviewBox';

/**
 * Client javascript holding events
 */
export class OctoClient {

    private static USER_ANNONYMOUS = 'annonymous';

    /**
     * Server Domain
     */
    private static BASE_DOMAIN = 'www.octopinion.co.uk';

    /**
    * Url for the server
    */
    private static URL = 'https://' + OctoClient.BASE_DOMAIN;

    /**
     * Url for socket
     */
    private static SOCKET_URL = 'wss://' + OctoClient.BASE_DOMAIN;

    /**
     * Token provided by Octopinion
     */
    private static TOKEN: string = '<%= token %>';

    /**
     * Username in Octopinion
     */
    private static USERNAME: string = '<%= user %>';

    /**
     * Identifier to be used in the socket connection
     */
    private identifier: number;

    /**
     * Web socket connection holder
     */
    private ws: WebSocket;

    /**
     * Review text 
     */
    private txt: string;

    /**
     * Section for this bunch of reviews 
     */
    private section = '';

    /**
     * User key identifying the user that has made the review.
     * If empty annonymous is used
     */
    private userKey = OctoClient.USER_ANNONYMOUS;

    /**
     * What to do when text has been analized.
     */
    private callback: any;

    /**
     * Element where the review box is inserted
     */
    private element: HTMLElement;

    /**
     * List of reviews
     */
    private reviews: Review[];

    /**
     * Tells if we want to insert last reviews
     */
    public insertLastReviews: boolean;

    /**
     * Tells if you want to show textarea only when userkey is set.
     */
    public hideWhenNotLogged = true;

    /**
     * Tells if you want to disable textarea only when userkey is set.
     */
    public disableWhenNotLogged = true;

    /**
     * Show more approach will render just a button to load more reviews.
     */
    public showMoreApproach = true;

    /**
     * Date format for reviews
     */
    public dateFormat: string = 'DD MMM YYYY';

    /**
     * Holds paging functionality
     */
    public octoPaging: OctoPaging;

    constructor(options: OctoOptions, token?: string, username?: string) {
        this.insertLastReviews = options.insertLastReviews;

        this.octoPaging = new OctoPaging(options.pageSize);
        this.identifier = parseInt((Math.random() * 100000).toString());
    }

    /**
     * Add a callback function to execute when text has been analized.
     * The function will receive a json data with the information about the analized.
     * 
     * interface AnalizedResponse {
     *  status: string;
     *  message: string;
     *  positiveFound: Founds[];
     *  negativeFound: Founds[];
     *  processedText: string;
     * }
     * 
     * @param callbackSuccess 
     */
    public setCallback(callbackSuccess: any) {
        this.callback = callbackSuccess;
    }

    public setText = (value: string) => {
        this.txt = value;
    }

    public getText = (): string => {
        return this.txt || '';
    }

    public setUserLoggedKey = (userKey: string) => {
        this.userKey = userKey;
    }

    public setPageSize = (pageSize: number) => {
        this.octoPaging.pageSize = pageSize;
    }

    public setInsertLastReviews = (insertLastReviews: boolean) => {
        this.insertLastReviews = insertLastReviews;
    }

    public setHideWhenNotLogged = (hideWhenNotLogged: boolean) => {
        this.hideWhenNotLogged = hideWhenNotLogged;
    }

    public setDisableWhenNotLogged = (disableWhenNotLogged: boolean) => {
        this.disableWhenNotLogged = disableWhenNotLogged;
    }

    public setDateFormat(format: string) {
        this.dateFormat = format;
    }

    public setShowMoreApproach(showMoreApproach: boolean) {
        this.showMoreApproach = showMoreApproach;
    }

    public setOptions(options: OctoOptions) {
        if (options.pageSize) this.octoPaging.pageSize = options.pageSize;
        if (options.insertLastReviews !== undefined) this.insertLastReviews = options.insertLastReviews;
        if (options.userKey) this.userKey = options.userKey;
        if (options.hideWhenNotLogged !== undefined) this.hideWhenNotLogged = options.hideWhenNotLogged;
        if (options.disableWhenNotLogged !== undefined) this.disableWhenNotLogged = options.disableWhenNotLogged;
        if (options.dateFormat !== undefined) this.dateFormat = options.dateFormat;
        if (options.showMoreApproach !== undefined) this.showMoreApproach = options.showMoreApproach;
    }

    /**
     * Generate a html and inserts in the element
     * 
     * @param element Html element where the reviews box is placed
     * @param insertLastReviews
     * @param section section where the review belong
     */
    public insertInto = (element: HTMLElement, section: string): void => {
        this._setUpSocketRealTimeRefresh();
        this.section = section;
        this.element = element;
        this._buildComponent(0);
    }

    private _buildComponent = (page: number): void => {
        if (this.insertLastReviews) {
            this.getAnalyzeds(page).subscribe(
                (data: AnalizedsPage) => {
                    this.octoPaging.currentPage = page;
                    this.octoPaging.lastPage = data.totalPages - 1;
                    if (!data.content) { data.content = [] }
                    this.reviews = data.content;
                    ReactDOM.render(<ReviewBox addMore={this.getMoreReviews} getReviewsForPage={this.getReviewsForPage} paging={this.octoPaging} setText={this.setText} analize={this.analize} data={data.content} showMoreApproach={this.showMoreApproach} dateFormat={this.dateFormat} hideTextArea={this._getHideTextarea()} disableTextArea={this._getDisableTextarea()} />, this.element);
                },
                error => console.error('Something went wrong', error)
            );
        } else {
            ReactDOM.render(<NewReviewBox  parent={undefined} getReviewsForPage={this.getReviewsForPage} disable={this._getDisableTextarea()} hide={this._getHideTextarea()} analize={this.analize} setText={this.setText} />, this.element);
        }
    }

    public getReviewsForPage = (page: number): Rx.Observable<AnalizedsPage> => {
        return this.getAnalyzeds(page)
                .retry(1)
                .map((data: AnalizedsPage) => {
                    this.octoPaging.currentPage = page;
                    this.octoPaging.lastPage = data.totalPages - 1;
                    this.reviews = data.content;
                    return data;
        });
    }

    public getMoreReviews = (): Rx.Observable<AnalizedsPage> => {
        if (this.insertLastReviews) {
            if (this.octoPaging.currentPage >= this.octoPaging.lastPage) {
               throw new Error('No more pages to render');
            }
            return this.getAnalyzeds(this.octoPaging.currentPage + 1)
                .retry(1)
                .map((data: AnalizedsPage) => {
                    this.octoPaging.currentPage++;
                    this.octoPaging.lastPage = data.totalPages - 1;
                    this.reviews = this.reviews.concat(data.content);
                    return data;
                });
        } else {
            console.error('Trying to add more reviews but insertLastReviews options is disabled');
        }
    }

    private _setUpSocketRealTimeRefresh = (): void => {
        this.ws = new WebSocket(OctoClient.SOCKET_URL + '/refresh');
        this.ws.onmessage = this._buildComponent.bind(this, 0);
        setTimeout(() => { this._subscriveToServer(OctoClient.USERNAME + '_npm_', this.identifier); }, 1000);
    }

    private _subscriveToServer = (user: string, identifier: number) => {
        if (!this.ws) {
            console.error('Web socket not established');
        } else {
            this.ws.send((new ConnectionParams(user, identifier)).toJson());
        }
    }

    private _getHideTextarea = (): boolean => {
        let hide: boolean;

        if (this.hideWhenNotLogged) {
            hide = this.userKey === OctoClient.USER_ANNONYMOUS;
        } else {
            hide = false;
        }
        return hide;
    }

    private _getDisableTextarea = (): boolean => {
        let disable: boolean;

        if (this.disableWhenNotLogged) {
            disable = this.userKey === OctoClient.USER_ANNONYMOUS;
        } else {
            disable = false;
        }
        return disable;
    }

    public getAnalyzeds = (page: number): Rx.Observable<AnalizedsPage> => {
        return Rx.Observable.ajax.getJSON(`${OctoClient.URL}/reviews/${page}/${this.octoPaging.pageSize}/${this.section}/${OctoClient.USERNAME}/?`
            + qs.stringify({ token: OctoClient.TOKEN }))
    }

    /**
     * Analize the review  in the #this.txt and rebuilds compoenent in the first page.
     */
    public analize = (): Rx.Observable<AnalizedResponse> => {

        if (this._getDisableTextarea() || this._getHideTextarea()) { throw new Error('You can\'t send reviews. Please log in!'); }

        return Rx.Observable.ajax.getJSON(`${OctoClient.URL}/evaluate/${this.section}/${OctoClient.USERNAME}/?` +
            qs.stringify({ text: this.txt, token: OctoClient.TOKEN, userkey: this.userKey, identifier: this.identifier }))
            .retry(1)
            .map(data => {
                if((data as AnalizedResponse).status === 'OK' && this.insertLastReviews)this._buildComponent(0);
                if ((data as AnalizedResponse).status === 'OK' && this.callback) {
                    this.callback.call(this, data);
                }
                return data as AnalizedResponse;
            });
    };
}

var octoClient = new OctoClient({ insertLastReviews: true, pageSize: 20, hideWhenNotLogged: true, disableWhenNotLogged: true });

/**
 * Make it available in window
 */
try {
    (window as any).octoClient = octoClient;
} catch (e) {
    console.warn('Window is not defined')
}

export default OctoClient;