var chai = require('chai');
var should = chai.should();
var expect = chai.expect;
var sinon = require('sinon');
var ReactTestUtils = require('react-dom/test-utils');
var $ = require('./jsdomjquery_setup').default;
var OctoClient = require('../dist/octoClient');
var Rx = require('rxjs');

/**
 * stub for websocket
 */
const socketSetUpSpy = sinon.stub(window.octoClient, '_setUpSocketRealTimeRefresh');

/**
 * Spying the private _buildComponent
 */
const buildComponentSpy = sinon.spy(window.octoClient, '_buildComponent');

/**
 * Mocks getAnalyzeds method and checks it has been called with expectedPage and returns mock response with first and last flags.
 * 
 * @param expectedPage expected page to retrieve
 * @param first if is the first page 
 * @param last if is the last page
 */
function buildGetAnalyzeds(expectedPage, first, last) {
    window.octoClient.getAnalyzeds = function (page) {
            expect(page).to.be.eq(expectedPage);
            return Rx.Observable.of({
                    first: first, last: last, number: 2, numberOfElements: 2, size: 2, totalElements: 8, totalPages: 4,
                    content: [{ id: 'id_1', processedtext: 'processed_1', creationDate: new Date(), userKey: 'user_key_1' },
                    { id: 'id_2', processedtext: 'processed_2', creationDate: new Date(), userKey: 'user_key_1' }
                    ]
                });
    };
};

describe('Octo Client', function () {

    beforeEach(function() {
        socketSetUpSpy.reset();
        buildComponentSpy.reset();
    });

    it('Sets default values correctly', function (done) {
        expect(window.octoClient.insertLastReviews).to.be.true;
        expect(window.octoClient.octoPaging.pageSize).to.be.eq(20);
        expect(window.octoClient.hideWhenNotLogged).to.be.true;
        expect(window.octoClient.disableWhenNotLogged).to.be.true;
        expect(window.octoClient.showMoreApproach).to.be.true;
        expect(window.octoClient.dateFormat).to.be.eq('DD MMM YYYY');
        expect(window.octoClient.userKey).to.be.eq('annonymous');
        expect(window.octoClient.callback).to.be.undefined;
        expect(window.octoClient.section).to.be.eq('');
        expect(window.octoClient.txt).to.be.undefined;
        expect(window.octoClient.identifier).to.be.not.undefined;

        done();
    });

    it('Changes default values when setOptions called', function (done) {
        window.octoClient.setOptions({ pageSize: 5, insertLastReviews: false, userKey: 'my_user', hideWhenNotLogged: false, disableWhenNotLogged: false, dateFormat: 'YYYY-MM-DD', showMoreApproach: false })
        expect(window.octoClient.insertLastReviews).to.be.false;
        expect(window.octoClient.octoPaging.pageSize).to.be.eq(5);
        expect(window.octoClient.hideWhenNotLogged).to.be.false;
        expect(window.octoClient.disableWhenNotLogged).to.be.false;
        expect(window.octoClient.showMoreApproach).to.be.false;
        expect(window.octoClient.dateFormat).to.be.eq('YYYY-MM-DD');
        expect(window.octoClient.userKey).to.be.eq('my_user');
        expect(window.octoClient.callback).to.be.undefined;
        expect(window.octoClient.section).to.be.eq('');
        expect(window.octoClient.txt).to.be.undefined;
        expect(window.octoClient.identifier).to.be.not.undefined;

        done();
    });

    it('Get analyzeds builds with last reviews and a logged user in the first page', function (done) {

        const divEl = global.document.createElement("div");
        window.octoClient.setOptions({ insertLastReviews: true, userKey: 'some_user', pageSize: 2, showMoreApproach: false});

        buildGetAnalyzeds(0, true, false);

        window.octoClient.insertInto(divEl, 'section');
        setTimeout(function(){   
            expect(socketSetUpSpy.calledOnce).to.be.true;
            expect(window.octoClient.section).to.be.eq('section');
            expect(buildComponentSpy.calledOnce).to.be.true;
            expect( $(divEl).find('textarea')[0] ).to.exist;
            expect( $(divEl).find('.fa-paper-plane-o')[0] ).to.exist;
            expect( $(divEl).find('.fa-fast-forward')[0] ).to.exist;
            expect( $(divEl).find('.fa-forward')[0] ).to.exist;
            expect( $(divEl).find('.fa-backward')[0] ).to.be.undefined;
            expect( $(divEl).find('.fa-fast-backward')[0] ).to.be.undefined;
            expect( $(divEl).find('.comment').length ).to.be.eq(2);
            
            done();
        }, 200);
    });

    it('Get analyzeds builds with last reviews and a logged user in the second page', function (done) {

        const divEl = global.document.createElement("div");
        window.octoClient.setOptions({ insertLastReviews: true, userKey: 'some_user', pageSize: 2, showMoreApproach: false});

        buildGetAnalyzeds(1, false, false);

        window.octoClient.element = divEl;
        window.octoClient.section = 'section';

        window.octoClient._buildComponent(1);

        setTimeout(function(){   
            expect(window.octoClient.section).to.be.eq('section');
            expect(buildComponentSpy.calledOnce).to.be.true;
            expect( $(divEl).find('textarea')[0] ).to.exist;
            expect( $(divEl).find('.fa-paper-plane-o')[0] ).to.exist;
            expect( $(divEl).find('.fa-fast-forward')[0] ).to.exist;
            expect( $(divEl).find('.fa-forward')[0] ).to.exist;
            expect( $(divEl).find('.fa-backward')[0] ).to.be.undefined;
            expect( $(divEl).find('.fa-fast-backward')[0] ).to.exist;
            expect( $(divEl).find('.comment').length ).to.be.eq(2);
            
            done();
        }, 200);
    });

    it('Get analyzeds builds with last reviews and a logged user in the last page', function (done) {

        const divEl = global.document.createElement("div");
        window.octoClient.setOptions({ insertLastReviews: true, userKey: 'some_user', pageSize: 2, showMoreApproach: false});

        buildGetAnalyzeds(3, false, true);

        window.octoClient.element = divEl;
        window.octoClient.section = 'section';

        window.octoClient._buildComponent(3);

        setTimeout(function(){   
            expect(window.octoClient.section).to.be.eq('section');
            expect(buildComponentSpy.calledOnce).to.be.true;
            expect( $(divEl).find('textarea')[0] ).to.exist;
            expect( $(divEl).find('.fa-paper-plane-o')[0] ).to.exist;
            expect( $(divEl).find('.fa-fast-forward')[0] ).be.undefined;
            expect( $(divEl).find('.fa-forward')[0] ).to.be.undefined;
            expect( $(divEl).find('.fa-backward')[0] ).to.exist;
            expect( $(divEl).find('.fa-fast-backward')[0] ).to.exist;
            expect( $(divEl).find('.comment').length ).to.be.eq(2);
            
            done();
        }, 200);
    });

     it('Get analyzeds builds with last reviews and a logged user in the first page with showmore activated', function (done) {

        const divEl = global.document.createElement("div");
        window.octoClient.setOptions({ insertLastReviews: true, userKey: 'some_user', pageSize: 2, showMoreApproach: true});

        buildGetAnalyzeds(0, true, false);

        window.octoClient.element = divEl;
        window.octoClient.section = 'section';

        window.octoClient._buildComponent(0);

        setTimeout(function(){   
            expect(window.octoClient.section).to.be.eq('section');
            expect(buildComponentSpy.calledOnce).to.be.true;
            expect( $(divEl).find('textarea')[0] ).to.exist;
            expect( $(divEl).find('.fa-paper-plane-o')[0] ).to.exist;
            expect( $(divEl).find('.fa-fast-forward')[0] ).be.undefined;
            expect( $(divEl).find('.fa-forward')[0] ).to.be.undefined;
            expect( $(divEl).find('.fa-backward')[0] ).to.be.undefined;
            expect( $(divEl).find('.fa-fast-backward')[0] ).to.be.undefined;
            expect( $(divEl).find('.fa-refresh')[0] ).to.exist;
            
            expect( $(divEl).find('.comment').length ).to.be.eq(2);
            
            done();
        }, 200);
    });

    it('Get analyzeds builds without last reviews', function (done) {

        const divEl = global.document.createElement("div");
        window.octoClient.setOptions({ insertLastReviews: false, userKey: 'some_user', pageSize: 2, showMoreApproach: true});

        buildGetAnalyzeds(0, true, false);

        window.octoClient.element = divEl;
        window.octoClient.section = 'section';

        window.octoClient._buildComponent(0);

        setTimeout(function(){   
            expect(window.octoClient.section).to.be.eq('section');
            expect(buildComponentSpy.calledOnce).to.be.true;
            expect( $(divEl).find('textarea')[0] ).to.exist;
            expect( $(divEl).find('.fa-paper-plane-o')[0] ).to.exist;
            expect( $(divEl).find('.fa-fast-forward')[0] ).to.be.undefined;
            expect( $(divEl).find('.fa-forward')[0] ).to.be.undefined;
            expect( $(divEl).find('.fa-backward')[0] ).to.be.undefined;
            expect( $(divEl).find('.fa-fast-backward')[0] ).to.be.undefined;
            expect( $(divEl).find('.fa-refresh')[0] ).to.be.undefined;
            
            expect( $(divEl).find('.comment').length ).to.be.eq(0);
            
            done();
        }, 200);
    });
});